unit uMap;

interface

uses
	Windows, Classes, SysUtils;

const
	miOut = 1;
	miRoad = 2;
	miSafeTrack = 3;
	miStart = 4;
	miFinish = 5;

type
	TMapItem = integer;
	TMapException = class(Exception);

	TMapArrayType = array of array of TMapItem;
	
	TMapArray = class
	private
		fArr:TMapArrayType;
		fX,fY:integer;
		function GetMap(x, y: integer): integer;
		procedure SetMap(x, y: integer; const Value: integer);
	public
		constructor Create(nX,nY:integer);
		destructor Destroy;override;
		procedure SetSize(newX,newY:integer);
		procedure FreeMap;

		property Width:integer read fX;
		property Height:integer read fY;
		property DirectMap:TMapArrayType read fArr write fArr;
		property Map[x,y:integer]:integer read GetMap write SetMap;default;
	end;
	
	TMap = class
	private
		fMap:TMapArray;
		fStartX,fStartY:integer;
		procedure LoadMap(const fileName:string);
		function TranslateItem(ch:char):TMapItem;
		
		function GetDirectMap: TMapArrayType;
		function GetHeight: integer;
		function GetWidth: integer;
    function GetMap(x, y: integer): byte;
	public
		constructor Create(const fileName:string);
		destructor Destroy;override;

		property Map[x,y:integer]:byte read GetMap;default;
		property Width:integer read GetWidth;
		property Height:integer read GetHeight;
		property DirectMap:TMapArrayType read GetDirectMap;

		property StartX:integer read fStartX;
		property StartY:integer read fStartY;
	end;

const
	MapItemChars : array [miOut..miFinish] of char=('x','.',':','s','c');

implementation

//******************************************************************************
//******************************************************************************
{ TMap }
//******************************************************************************
//******************************************************************************
constructor TMap.Create(const fileName: string);
begin
	fMap:=TMapArray.Create(0,0);
	try
		LoadMap(fileName);
	except
		freeAndNil(fMap);
		raise;
	end;
end;

//******************************************************************************
//******************************************************************************
destructor TMap.Destroy;
begin
	FreeAndNil(fMap);
	inherited;
end;

//******************************************************************************
//******************************************************************************
function TMap.GetDirectMap: TMapArrayType;
begin
	result:=fMap.DirectMap;
end;

//******************************************************************************
//******************************************************************************
function TMap.GetHeight: integer;
begin
	result:=fMap.Height;
end;

//******************************************************************************
//******************************************************************************
function TMap.GetMap(x, y: integer): byte;
begin
	result:=fMap[x,y];
end;

//******************************************************************************
//******************************************************************************
function TMap.GetWidth: integer;
begin
	result:=fMap.Width;
end;

//******************************************************************************
//******************************************************************************
procedure TMap.LoadMap(const fileName: string);
var
	foundFinish:boolean;

	procedure ParseLine(const line:string;rowNum:integer);
	var
		i:integer;
		b:byte;
	begin
		if length(line)<>GetWidth then
			raise TMapException.CreateFmt('Nekonzistentn mapa (dlka dku: %d, oekvno %d)',[length(line),GetWidth]);
		for i:=1 to length(line) do
			try
				b:=TranslateItem(line[i]);
				if b=miFinish then
					foundFinish:=true;
				if b=miStart then begin
					if fStartX<>-1 then
						raise TMapException.Create('Dva starty jak cyp!');
					fStartX:=i-1;
					fStartY:=rowNum;
					fMap[i-1,rowNum]:=miRoad;
				end else
					fMap[i-1,rowNum]:=b;
			except
				on E:Exception do
					raise TMapException.CreateFmt('%s'#13#10'sloupec %d;',[E.Message,i]);
			end;
	end;
	
var
	sl:TStringList;
	i:integer;
	x,y:integer; // predpokladana velikost mapy
begin
	sl:=TStringList.Create;
	try
		sl.LoadFromFile(fileName);
		
		y:=sl.Count;
		if not(y in [1..255]) then
			raise TMapException.CreateFmt('Nesprvn velikost mapy (%d)',[y]);
			
		x:=length(sl[0]);
		if not (y in [1..255]) then
			raise TMapException.CreateFmt('Nesprvn velikost mapy (%d)',[x]);

		fMap.SetSize(x,y);
		fStartX:=-1;
		fStartY:=-1;
		foundFinish:=false;

		for i:=0 to y-1 do
			try
				parseLine(sl[i],i);
			except
				on E:Exception do
					raise TMapException.CreateFmt('%s'#13#10'dek %d;',[E.Message,i+1]);
			end;

		if fStartX=-1 then
			raise TMapException.Create('He, start neni pyo, he');

		if not foundFinish then
			raise TMapException.Create('He, nejni cil jak cyp');
				
	finally
		sl.free;
	end;
end;

//******************************************************************************
//******************************************************************************
function TMap.TranslateItem(ch: char): TMapItem;
var
	i:integer;
begin
	ch:=lowercase(ch)[1];
	for i:=low(MapItemChars) to high(MapItemChars) do
		if MapItemChars[i]=ch then begin
			Result:=i;
			exit;
		end;

	raise TMapException.CreateFmt('Neznm znak v map (%s)',[ch]);
end;

//******************************************************************************
//******************************************************************************
{ TMapArray }
//******************************************************************************
//******************************************************************************
constructor TMapArray.Create(nX, nY: integer);
begin
	SetSize(nX,nY);
end;

//******************************************************************************
//******************************************************************************
destructor TMapArray.Destroy;
begin
	FreeMap;
	inherited;
end;

//******************************************************************************
//******************************************************************************
procedure TMapArray.FreeMap;
var
	i:integer;
begin
	for i:=0 to fX-1 do
		SetLength(fArr[i],0);
	SetLength(fArr,0);
	fX:=0;
	fY:=0;
end;

//******************************************************************************
//******************************************************************************
function TMapArray.GetMap(x, y: integer): integer;
begin
	if (x in [0..fX-1]) and (y in [0..fY-1]) then
		result:=fArr[x,y]
	else
		result:=-1;
end;

//******************************************************************************
//******************************************************************************
procedure TMapArray.SetMap(x, y: integer; const Value: integer);
begin
	if (x in [0..fX-1]) and (y in [0..fY-1]) then
		fArr[x,y]:=value;
end;


//******************************************************************************
//******************************************************************************
procedure TMapArray.SetSize(newX, newY: integer);
var
	i:integer;
begin
	SetLength(fArr,newX);
	for i:=0 to newX-1 do
		SetLength(fArr[i],newY);
	fX:=newX;
	fY:=newY;
end;

end.
